// File:  box.cc
//
//   Example and the first try at a Graphic Class
//
#include <grx.h>
#include <mousex.h>
#include <iostream.h>    
#include <Strclass.h>
#include <fstream.h>
#include <stdio.h>
#define FAIL 0
#define SUCESS 1
//
// defaults to 1280 x 960 with 256 colors and in reality will
// mostly be less.
//

class Graphic
{       protected:

	static int initialized;
	static int ccount;
	static int MaxX,MaxY,MaxC;
	static float sscale;  
	// ratio of realpixel / MAX_Y pixels
	GrContext *ccontext;
	
	public:
	
	Graphic()
	{  if (initialized==0)
	   {    GrSetMode(GR_biggest_noninterlaced_graphics); 
		initialized=1;
		MaxX=GrMaxX();
		MaxY=GrMaxY();
		MaxC=GrNumColors();
		ccontext=NULL;
	   }
	   ccount++;

	}
	~Graphic()
	{  ccount--;
	   if (ccount==0)
		GrSetMode(GR_default_text);
	}
	inline int total_number_graphic_objects() { return ccount; };
	inline float scale() { return sscale; };
	inline void set_context(GrContext *new_context)
		{ ccontext=new_context; };
	inline GrContext* context() {return ccontext; };
	inline int maxx() { return MaxX; };
	inline int maxy() { return MaxY; };
	inline int maxc() { return MaxC; };
};
int Graphic::ccount=0;
int Graphic::initialized=0;
int Graphic::MaxX=0, Graphic::MaxY=0, Graphic::MaxC=0;
float Graphic::sscale=0;
#define MAX_COLORS 254
struct color_info
{       String color_name;
	int color_number;
} ;
class Color_Table : public Graphic
{
	private :
	   color_info c_table[ MAX_COLORS ] ;
	   int entries;
	public :
	   Color_Table():
	       Graphic(),
	       entries(0)
	   { c_table[entries].color_name="WHITE";
	     c_table[entries].color_number=GrWhite();
	     entries++;
	     c_table[entries].color_name="BLACK";
	     c_table[entries].color_number=GrBlack();
	     entries++;
	     for (int i=entries ; i<GrNumColors(); i++)
	     {   c_table[i].color_name="";
		 c_table[i].color_number=-1;
	     }
	   }
	   int alloc(int r, int g, int b, String name) 
	   { 
	     if (entries>=MaxC) 
		return FAIL;
	     c_table[entries].color_number=GrAllocColor(r,g,b);
	     c_table[entries].color_name=name;
	     entries++;
	     return SUCESS;
	   }
	   int c(String name)
	   {    for (int i=0 ; i<entries; i++)
		{  if (c_table[i].color_name==name)
		     return c_table[i].color_number;
		}
		return -1;
	   }
};
//
//      >> Operator to read Color Tables.
//
istream& operator>> (istream& s, Color_Table& c)
{       String name,brace;
	int r,g,b;

	s >> name;
	if (name!="Color_Table")
		return s;
	s >> brace;
	if (brace!="{")
		return s;
	do 
	{ s >> r >> g >> b >> name;
	  c.alloc(r,g,b,name);
	} while (name!="END");
	s >> brace;
	return s;
}

class Box : public Graphic
{ 
	protected:
		int cc;  
		int xx1, yy1, xx2, yy2;
	public:
		friend ostream& operator<< (ostream& s, const Box& b) ;
		Box() :
		  Graphic()
		{ xx1=0; yy1=0;
		  xx2=MaxX/12; yy2=MaxY/8;
		  cc=GrWhite(); 
		}
		Box(int x1, int y1, int x2, int y2, int c) :
		  Graphic()
		{ xx1=x1; yy1=y1; xx2=x2; yy2=y2; cc=c; }
		void cdraw()
		{ GrSetContext(ccontext);
		  GrBox(xx1,yy1,xx2,yy2,cc);
		}
		inline void fdraw() { GrBox(xx1,yy1,xx2,yy2,cc); };
		inline void set_c(int new_c) {cc=new_c;};
		inline int  c() { return cc; };

};
//
//      << Operator to print Box in a readable manner.
//
ostream& operator<< (ostream& s, const Box& b)
{       return s << "Box " << endl 
		 << b.xx1 << b.yy1 << b.xx2 
		 << b.yy2 << b.cc << endl;
}
#define Font_Hight 16
#define Font_Width 8
class Color_Selector : public Box
{
	protected:
		GrFBoxColors c1,c2;
		int color_cell;
		int r,g,b,w;
		int xx3,yy3,xx4,yy4;
		int bg;
		int top_left1x, top_left2x, top_left3x;
		int top_lefty;
	public :
		void setup();
		Color_Selector() : 
		  Box()
		  { this.setup(); }
		Color_Selector(int x1, int y1, int x2, int y2, int c) :
		  Box(x1,y1,x2,y2,c)
		  { this.setup(); }
		void cdraw()
		{ char ntext[4];  // Text Buffer for printing color values
				  // on graphic screen.
		  GrSetContext(ccontext);
		  GrFramedBox(xx1,yy1,xx2,yy2,w,&c1); // Draw Top Half
		  GrFramedBox(xx3,yy3,xx4,yy4,w,&c2); // Bottom Half
		  sprintf(ntext,"%d\0",r);  // Red index
		  GrTextXY(top_left1x,top_lefty,ntext,cc,bg);
		  sprintf(ntext,"%d",g);  // Green index
		  GrTextXY(top_left2x,top_lefty,ntext,cc,bg);
		  sprintf(ntext,"%d",b);  // Blue index
		  GrTextXY(top_left3x,top_lefty,ntext,cc,bg);
		}
		void fdraw() { this.cdraw(); };
	
};
void Color_Selector::setup() 
  {  r=g=b=125;       // Starting Color Values
     w=(xx2-xx1)/20;  // Thickness of borders 1/20 
		      // of width of box.
     // Make Sure that lower half is large enough to 
     // hold characters to display color values.
     // using 2*Font_Hight by 13*Font_Width
     // as min comforable space.
     xx3=xx1;
     yy3=yy2+w;
     yy4=yy3+2*Font_Hight;
     xx4=xx3+13*Font_Width;
     if (xx4<xx2) 
	xx4=xx2;   // use xx2 if its bigger

     // Top Left x corner of diplayed text fields
     top_left1x=xx1+Font_Width;
     top_left2x=top_left1x+Font_Width*4;
     top_left3x=top_left2x+Font_Width*4;

     // Top Left y is same for all 3 fields.
     top_lefty=yy3+Font_Hight/2;

     // Top Half of Color Selector Colors
     color_cell=GrAllocCell(); // variable color cell
     c1.fbx_intcolor=color_cell;  
     GrSetColor(color_cell,r,g,b);  // set the color
     c1.fbx_topcolor=cc;    c1.fbx_rightcolor=cc;
     c1.fbx_bottomcolor=cc; c1.fbx_leftcolor=cc;

     // Bottom Half of Color Selector Colors
     c2.fbx_intcolor=GrBlack();
     bg=c2.fbx_intcolor;
     c2.fbx_topcolor=cc;    c2.fbx_rightcolor=cc;
     c2.fbx_bottomcolor=cc; c2.fbx_leftcolor=cc;
  }

Color_Table c;  
main ()
{ Box b;
  ifstream color_file("COLOR.DAT");

  if (!color_file) 
  { c.alloc(120,0,0,"RED");
    b.set_c(c.c("RED"));
    b.fdraw();
  }
  else
  { color_file>>c;
    b.set_c(c.c("BLUE"));
    b.fdraw();
  }
  getkey();
/*  Color_Selector selector(c.maxx()/4,    // x1
			  c.maxy()/4,    // y1
			  c.maxx()/4*3,  // x2
			  c.maxy()/4*3,  // y2
			  c.c("WHITE")); // c
  selector.cdraw();
  getkey();
*/  Color_Selector selector2(c.maxx()/10,   // x1
			  c.maxy()/10,    // y1
			  c.maxx()/10*2,  // x2
			  c.maxy()/10*2,  // y2
			  c.c("RED"));    // c
  selector2.cdraw();
  getkey();
}
