// Graphic.h
#define FAIL 0
#define SUCESS 1
#include <grx.h>
#include <mousex.h>
#include <iostream.h>    
#include <Strclass.h>
#include <fstream.h>
/*****************************************************************************
 *                                                                           *
 *  Graphic Class                                                            *
 *                                                                           *
 *      Functions                                                            *
 *              Graphic()   - Constructor initalizes graphics.               *
 *              ~Graphic()  - Destructor restores text mode.                 *
 *                                                                           *
 *              total_number_graphic_objects()  - returns the number of      *
 *                              graphic objects defined                      *
 *              scale()     - scale, returns the scale number. (not used yet)*
 *                                                                           *
 *              set_context(GrContext *new_context) - Sets a new graphic     *
 *                              context for the graphic object               *
 *              context()   - returns the graphic context of the object      *
 *              maxx()      - returns the max number of pixels in the x dir  *
 *              maxy()      - returns the max number of pixels in the y dir  *
 *              maxc()      - returns the max number of colors allowed       *
 *                                                                           *
 *****************************************************************************/

class Graphic
{       protected:

        static int initialized;
        static int ccount;
        static int MaxX,MaxY,MaxC;
        GrContext *ccontext;
        
        public:
        
        Graphic()
        {  if (initialized==0)
           {    GrSetMode(GR_biggest_noninterlaced_graphics); 
                initialized=1;
                MaxX=GrMaxX();
                MaxY=GrMaxY();
                MaxC=GrNumColors();
           }
           ccontext=NULL;
           ccount++;

        }
        ~Graphic()
        {  ccount--;
           if (ccount==0)
                GrSetMode(GR_default_text);
        }
        inline int total_number_graphic_objects() { return ccount; };
        inline void set_context(GrContext *new_context)
                { ccontext=new_context; };
        inline GrContext* context() {return ccontext; };
        inline int maxx() { return MaxX; };
        inline int maxy() { return MaxY; };
        inline int maxc() { return MaxC; };
};

int Graphic::ccount=0;
int Graphic::initialized=0;
int Graphic::MaxX=0, Graphic::MaxY=0, Graphic::MaxC=0;

#define MAX_COLORS 254
struct color_info
{       String color_name;
	int color_number;
} ;
class Color_Table : public Graphic
{
	private :
	   color_info c_table[ MAX_COLORS ] ;
	   int entries;
	public :
	   Color_Table():
	       Graphic(),
	       entries(0)
	   { c_table[entries].color_name="WHITE";
	     c_table[entries].color_number=GrWhite();
	     entries++;
	     c_table[entries].color_name="BLACK";
	     c_table[entries].color_number=GrBlack();
	     entries++;
	     for (int i=entries ; i<GrNumColors(); i++)
	     {   c_table[i].color_name="";
		 c_table[i].color_number=-1;
	     }
	   }
	   int alloc(int r, int g, int b, String name) 
	   { 
	     if (entries>=MaxC) 
		return FAIL;
	     c_table[entries].color_number=GrAllocColor(r,g,b);
	     c_table[entries].color_name=name;
	     entries++;
	     return SUCESS;
	   }
	   int c(String name)
	   {    for (int i=0 ; i<entries; i++)
		{  if (c_table[i].color_name==name)
		     return c_table[i].color_number;
		}
		return -1;
	   }
};
//
//      >> Operator to read Color Tables.
//
istream& operator>> (istream& s, Color_Table& c)
{       String name,brace;
	int r,g,b;

	s >> name;
	if (name!="Color_Table")
		return s;
	s >> brace;
	if (brace!="{")
		return s;
	do 
	{ s >> r >> g >> b >> name;
	  c.alloc(r,g,b,name);
	} while (name!="END");
	s >> brace;
	return s;
}

class Box : public Graphic
{ 
	protected:
		int cc;  
		int xx1, yy1, xx2, yy2;
	public:
		friend ostream& operator<< (ostream& s, const Box& b) ;
		Box() :
		  Graphic()
		{ xx1=0; yy1=0;
		  xx2=MaxX/12; yy2=MaxY/8;
		  cc=GrWhite(); 
		}
		Box(int x1, int y1, int x2, int y2, int c) :
		  Graphic()
		{ xx1=x1; yy1=y1; xx2=x2; yy2=y2; cc=c; }
		void cdraw()
		{ GrSetContext(ccontext);
		  GrBox(xx1,yy1,xx2,yy2,cc);
		}
		inline void fdraw() { GrBox(xx1,yy1,xx2,yy2,cc); };
		inline void set_c(int new_c) {cc=new_c;};
		inline int  c() { return cc; };
		void add_x( int added_x) ;
		void add_y( int added_y) ;
};
//
//      << Operator to print Box in a readable manner.
//
ostream& operator<< (ostream& s, const Box& b)
{       return s << "Box " << endl 
		 << b.xx1 << b.yy1 << b.xx2 
		 << b.yy2 << b.cc << endl;
}
void Box::add_x(int added_x) 
{       xx2+=added_x;
	this.fdraw();      
}
void Box::add_y(int added_y) 
{       yy2+=added_y;
	this.fdraw();
}
