// file : Position.h
//
// Example of a complete class
//
#include <iostream.h>
//
//  Stage
//      An enumerated type to describe being on stage ie: centered from
//      front to rear, vs background and forground.  Also has no_stage
//      meaning that no value for the variable has been assigned yet or
//      that there is no valid current value.
//
enum   stage 
	{       no_stage,
		forground,
		on_stage,
		background
	};
//
//      << Operator for the stage enumerated type
//
//      Prints the stage variable in a readable manner.
//
ostream& operator<< (ostream& s, const stage& stg);


//      where
//         An enumerated type describing where the reference is taken
//      from. Ie from the screen or from the map. Also has the value
//      no_where for an invalid or uninitialized value.
//      
enum    where
	{       no_where,
		map,
		screen
	};
//
//      << operator for where enumerated type
//
//      Prints the where variable in a readable manner.
//
ostream& operator<< (ostream& s, const where& w);

//
//      Position
//         Uses the where, and stage enumerated type to store 
//      position information for an object on the map and or the
//      screen.
//
class Position
{       
	int xx,yy;       // x and y coordinate numbers
	stage zz;        // depth ie: a stage
	where ww;        // on map or on screen

	public:
	// Access Functions
	//        
	//
	int x() { return ((int) xx); }
	int y() { return ((int) yy); }
	where w() { return ((where) ww); }
	stage z() { return ((stage) zz); }
	friend ostream& operator<< (ostream& s, const Position& p);
	
	// Access Functions
	//        to set or zero out the object
	//
	void zero();
	void set(const int new_x, const int new_y);
	void set(const int new_x, const int new_y, 
			 const stage new_z, const where new_w );
	void set_x(const int new_x) { xx=(int) new_x; }
	void set_y(const int new_y) { yy=(int) new_y; }
	void set_w(const int new_w) { ww=(where) new_w; }
	void set_z(const int new_z) { zz=(stage) new_z; }
	// Overloaded Constructors
	// for various amounts of information.

	Position() :
		xx(0) ,
		yy(0) ,
		ww(no_where), 
		zz(no_stage) 
		{}
	Position(int x, int y) :
		xx(x) ,
		yy(y) ,
		ww(no_where) ,
		zz(no_stage)
		{}
	Position(int x, int y, stage z , where w) :
		xx(x) ,
		yy(y) ,
		ww(w) ,
		zz(z)
		{}
	
	//      Assignment operator
	//
	Position& operator= (const Position& source);
	
	//      For Addition and Subtraction Assigments
	//
	Position& operator+= (const Position& source);
	Position& operator-= (const Position& source);
	Position operator* (const Position& source, const float &multiplier);
	Position operator/ (const Position& source, const float &divisor);


};
// Access funtions defined

inline void Position::zero() 
	{ xx=0; yy=0; };

inline void Position::set(const int new_x, const int new_y) 
	{ xx=new_x; yy=new_y;};

inline void Position::set(const int new_x, const int new_y, 
			  const stage new_z, const where new_w)
{       this.set(new_x, new_y); 
	zz=new_z; ww=new_w; 
};

// Operator functions for Postion defined
//
//      Assignment operator '='
//
inline Position& Position::operator= (const Position& source)
{       xx = source.xx;
	yy = source.yy;
	zz = source.zz;
	ww = source.ww;
	return *this;
};
//
//      Assginment and addition '+='
//
inline Position& Position::operator+= (const Position& source)
{       xx += source.xx;
	yy += source.yy;
	return *this;
};
//
//      Assignment and subtraction '-='
//
inline Position& Position::operator-= (const Position& source)
{       xx -= source.xx;
	yy -= source.yy;
	return *this;
};
//
//      << Operator for the stage enumerated type
//
//      Prints the stage variable in a readable manner.
//
ostream& operator<< (ostream& s, const stage& stg)
{       switch (stg)
	{       case no_stage   : return s << "no_stage";
		case forground  : return s << "forground";
		case on_stage   : return s << "on_stage";
		case background : return s << "background";
	}
}
//
//      << operator for where enumerated type
//
//      Prints the where variable in a readable manner.
//
ostream& operator<< (ostream& s, const where& w)
{       switch (w)
	{       case no_where   : return s << "no_where";
		case map        : return s << "map";
		case screen     : return s << "screen";
	}
} 
//
//      << Operator to print positions in a readable manner.
//
ostream& operator<< (ostream& s, const Position& p)
{       return s << "The Position is (" << p.xx << "," << p.yy << ")  "
		 << p.zz << " on the " << p.ww ;
}

Position operator* (const Position& source, const float &multiplier)
{	Position target;
	target=source;
	target.set_x(target.x()*multiplier);
	target.set_y(target.y()*multiplier);
	return target;
}
Position operator/ (const Position& source, const float &divisor)
{	Position target;
	target=source;
	target.set_x(target.x()/divisor);
	target.set_y(target.y()/divisor);
	return target;
}



