#define __WINDOW_CC
#include "window.h"
/***********************************************************************
 *  Window.cc                                                          *
 *                                                                     *
 *      This is a C++ Modual to support window oriented graphics       *
 *      for computer games.                                            *
 *                                                                     *
 ***********************************************************************/

/***********************************************************************
 *  Message Functions:                                                 *                  
 *          mess_i  -  Prints the string and integer value at top      *
 *                      left of graphics screen for debugging and      *
 *                      error message reporting                        *
 *                                                                     *
 *          mess_s  -  Prints a string and a C style character array   *    
 ***********************************************************************/

void mess_i(String m,int val)
{       char buffer[30];
        GrTextXY(0,0,"                                ",
                GrWhite(),GrBlack());
        sprintf(buffer,"%d\0",val);
        cat(m," ",buffer,m);
        GrTextXY(0,0,(char*)m,GrWhite(),GrBlack());
        getkey();
}   
void mess_s(String m,char *val)
{       cat(m," ",val,m);
        GrTextXY(0,0,"                                ",GrWhite(),GrBlack());
        GrTextXY(0,0,(char*)m,GrWhite(),GrBlack());
        getkey();
}
/***********************************************************************
 *                                                                     *
 *      Color Table Class Member Functions and Support Functions       *
 *                                                                     *
 *      operator>>      Operator to read Color Tables.                 *
 *      Color_Table()   Default Constructor.                           *
 *                                                                     *
 ***********************************************************************/
//
//      >> Operator to read Color Tables.
//
istream& operator>> (istream& s, Color_Table& c)
{       String name,brace;
        int r,g,b;

        s >> name;
        if (name!="Color_Table")
                return s;
        s >> brace;
        if (brace!="{")
                return s;
        do 
        { s >> r >> g >> b >> name;
          c.alloc(r,g,b,name);
        } while (name!="END");
        s >> brace;
        return s;
}
//
//      Color Table Constructor
//
Color_Table::Color_Table():
               Graphic(),
               entries(0)
{ c_table[entries].color_name="WHITE";
  c_table[entries].color_number=GrWhite();
  entries++;
  c_table[entries].color_name="BLACK";
  c_table[entries].color_number=GrBlack();
  entries++;
  for (int i=entries ; i<GrNumColors(); i++)
  {   c_table[i].color_name="";
      c_table[i].color_number=-1;
  }
}
/***********************************************************************
 *                                                                     *
 *      Box Class member functions and support functions               *
 *                                                                     *
 *      add_x(int added_x)      Add an integer to the x position       *  
 *                                      top left corner coordinate     *
 *      add_y(int added_y)      Add an integer to the y position       *                                       
 *      operator<<              Print box information                  *                                                 *
 *                                                                     *
 ***********************************************************************/
//
//      << Operator to print Box in a readable manner.
//
ostream& operator<< (ostream& s, const Box& b)
{       return s << "Box " << endl 
                 << b.xx1 << b.yy1 << b.xx2 
                 << b.yy2 << b.cc << endl;
}
void Box::add_x(int added_x) 
{       xx2+=added_x;
        this.fdraw();      
}
void Box::add_y(int added_y) 
{       yy2+=added_y;
        this.fdraw();
}
/***********************************************************************
 *                                                                     *
 *      Color Selector Class member functions and support functions    *
 *                                                                     *
 *       setup()               Called by constuctors for code that is  *
 *                              allways needed by each constructor     *
 *       cdraw()               Context Draw Function                   *
 *       add_x(int added_x)    Add to x coordinate & redraw            *
 *       add_y(int added_y)    Add to y coordinate & redraw            *
 *       add_r(int added_r)    Add to red value                        *
 *       add_g(int added_g)    Add to green value                      *
 *       add_b(int added_b)    Add to blue value                       *
 *       adjust()              Interactivly adjust colors              *
 *                                                                     *
 ***********************************************************************/
void Color_Selector::setup() 
  {  w=(xx2-xx1)/SPACING;  // Thickness of borders 1/20 
                           // of width of box.
     // Make Sure that lower half is large enough to 
     // hold characters to display color values.
     // using 2*Font_Hight by 13*Font_Width
     // as min comforable space.
     xx3=xx1;
     yy3=yy2+w;
     yy4=yy3+2*Font_Hight;
     xx4=xx3+13*Font_Width;
     if (xx4<xx2) 
        xx4=xx2;   // use xx2 if its bigger

     // Top Left x corner of diplayed text fields
     top_left1x=xx1+Font_Width;
     top_left2x=top_left1x+Font_Width*4;
     top_left3x=top_left2x+Font_Width*4;

     // Top Left y is same for all 3 fields.
     top_lefty=yy3+Font_Hight/2;

     // Top Half of Color Selector Colors
     color_cell=GrAllocCell(); // variable color cell
     c1.fbx_intcolor=color_cell;  
     GrSetColor(color_cell,r,g,b);  // set the color
     c1.fbx_topcolor=cc;    c1.fbx_rightcolor=cc;
     c1.fbx_bottomcolor=cc; c1.fbx_leftcolor=cc;

     // Bottom Half of Color Selector Colors
     c2.fbx_intcolor=GrBlack();
     bg=c2.fbx_intcolor;
     c2.fbx_topcolor=cc;    c2.fbx_rightcolor=cc;
     c2.fbx_bottomcolor=cc; c2.fbx_leftcolor=cc;
  }
  
void Color_Selector::cdraw()
  { char ntext[4];  // Text Buffer for printing color values
                   // on graphic screen.
    GrSetContext(ccontext);
    GrFramedBox(xx1,yy1,xx2,yy2,w,&c1); // Draw Top Half
    GrFramedBox(xx3,yy3,xx4,yy4,w,&c2); // Bottom Half
    sprintf(ntext,"%d\0",r);  // Red index
    GrTextXY(top_left1x,top_lefty,ntext,cc,bg);
    sprintf(ntext,"%d",g);  // Green index
    GrTextXY(top_left2x,top_lefty,ntext,cc,bg);
    sprintf(ntext,"%d",b);  // Blue index
    GrTextXY(top_left3x,top_lefty,ntext,cc,bg);
  }
void Color_Selector::add_x(int added_x)
  {  xx2+=added_x;
     this.setup();
     this.cdraw();
  }
void Color_Selector::add_y(int added_y)
  {  yy2+=added_y;
     this.setup();
     this.cdraw();
  }
void Color_Selector::add_r(int added_r)
  {  r+=added_r;
     if(r>255) r=255;    // enforce limits
     if(r<0) r=0;
     this.cset();
     this.cdraw();
  }
void Color_Selector::add_g(int added_g)
  {  g+=added_g;
     if(g>255) g=255;    // enforce limits
     if(g<0) g=0;
     this.cset();
     this.cdraw();
  }
void Color_Selector::add_b(int added_b)
  {  b+=added_b;
     if(b>255) b=255;    // enforce limits
     if(b<0) b=0;
     this.cset();
     this.cdraw();
  }
void Color_Selector::adjust()
  {     char c;
        c= getch();
        while (c!=(char) 27)
        {       // Red adjust keys
                if (c=='q') this.add_r(10);
                if (c=='w') this.add_r(1);
                if (c=='e') this.add_r(-1);
                if (c=='r') this.add_r(-10);
                // Green adjust keys
                if (c=='a') this.add_g(10);
                if (c=='s') this.add_g(1);
                if (c=='d') this.add_g(-1);
                if (c=='f') this.add_g(-10);
                // Blue adjust keys
                if (c=='z') this.add_b(10);
                if (c=='x') this.add_b(1);
                if (c=='c') this.add_b(-1);
                if (c=='v') this.add_b(-10);
                c=getch();
        }
  }

/***********************************************************************
 *                                                                     *
 *      Win Color structure support functions                          *
 *                                                                     *
 *       operator>>    Operator to read Win Colors                     *
 *                                                                     *
 ***********************************************************************/

//      >> Operator to read Win Colors.
//
istream& operator>> (istream& s, Win_Colors& Wcp)
{       String name,brace,color;

        s >> name;
        if (name!="Win_Colors")
                return s;
        s >> brace;
        if (brace!="{")
                return s;
do 
{ s >> name >> color;
  if (name=="background"     ) Wcp.background     =color_table.c(color);
  if (name=="window"         ) Wcp.window         =color_table.c(color);
  if (name=="window_frame"   ) Wcp.window_frame   =color_table.c(color);
  if (name=="char_in_win"    ) Wcp.char_in_win    =color_table.c(color);
  if (name=="scroll_bar"     ) Wcp.scroll_bar     =color_table.c(color);
  if (name=="menu"           ) Wcp.menu           =color_table.c(color);
  if (name=="char_in_menu"   ) Wcp.char_in_menu   =color_table.c(color);
  if (name=="active_title_bar") Wcp.active_title_bar=color_table.c(color);
  if (name=="inact_title_bar") Wcp.inact_title_bar=color_table.c(color);
  if (name=="char_act_title" ) Wcp.char_act_title =color_table.c(color);
  if (name=="char_ina_title" ) Wcp.char_ina_title =color_table.c(color);
  if (name=="popup_win"      ) Wcp.popup_win      =color_table.c(color);
  if (name=="char_in_popup"  ) Wcp.char_in_popup  =color_table.c(color);
  if (name=="message_window" ) Wcp.message_window =color_table.c(color);
  if (name=="char_in_message") Wcp.char_in_message=color_table.c(color);
  if (name=="button_face"    ) Wcp.button_face    =color_table.c(color);
  if (name=="button_text"    ) Wcp.button_text    =color_table.c(color);
  if (name=="button_shadow"  ) Wcp.button_shadow  =color_table.c(color);
  if (name=="button_highlight") Wcp.button_highlight=color_table.c(color);
} while (color!="END");
        s >> brace;
        return s;
}
/***********************************************************************
 *                                                                     *
 *      Title Bar Class member functions and support functions         *
 *                                                                     *
 *       locate(int x1,int y1,int x2,int y2)                           *
 *                                                                     *
 *      Note: will choose a value for x2 if x2=-1                      *
 *                                                                     *
 ***********************************************************************/
void Title_Bar::locate(int x1,int y1,int x2,int y2)
{  // Given windows coordinates change
   // title bar coordiantes to match.
   // and load appropreate font.
   int max_width,max_height;
   int tot_width,tot_height;
   int flag=0;
    
   if (x2==-1)
   { flag=1; x2=4096; }               
   xx1=x1+1;
   yy1=y1+1;
   xx2=x2-1;
   yy2=y1+1+FONTHEIGHT;
                   
   max_width=(xx2-xx1)/len;
   max_height=yy2-yy1;
   if(NULL==GrFindBestFont(max_width,
                           max_height,
                           0,
                           (char *) family,
                           &op )
     )
   mess_i("Null returned from GrFind",0);
   op.txo_fgcolor.v=Wcp.char_act_title;
   op.txo_bgcolor.v=Wcp.active_title_bar;
   tot_width=GrStringWidth((char *)message,len,&op);
   tot_height=GrStringHeight((char *)message,len,&op);
   if (flag==1)
   { xx2=xx1+tot_width+tot_width/SPACING; }
   m_x=xx1+(xx2-xx1-tot_width)/2;
   m_y=yy1+(yy2-yy1-tot_height)/2;
}
/***********************************************************************
 *                                                                     *
 *      Window Class member functions and support functions            *
 *                                                                     *
 *      resize()        Interactivly resize window                     *
 *      move()          Interactivly move window                       *
 *      add_xx1(int x)  Add integer to x coordinate                    *                           *
 *      add_yy1(int y)  Add integer to y coordinate                    *
 *                                                                     *
 ***********************************************************************/
void Window::resize()
{ char c;
  c= getch();
  while (c!=(char) 27)
  {     // X adjust keys
        if (c=='q') this.add_x(10);
        if (c=='w') this.add_x(1);
        if (c=='e') this.add_x(-1);
        if (c=='r') this.add_x(-10);
        // Y adjust keys
        if (c=='a') this.add_y(10);
        if (c=='s') this.add_y(1);
        if (c=='d') this.add_y(-1);
        if (c=='f') this.add_y(-10);
        c=getch();
  }
}
void Window::move()
{ char c;
  c= getch();
  while (c!=(char) 27)
  {     // X adjust keys
        if (c=='q') this.add_xx1(10);
        if (c=='w') this.add_xx1(1);
        if (c=='e') this.add_xx1(-1);
        if (c=='r') this.add_xx1(-10);
        // Y adjust keys
        if (c=='a') this.add_yy1(10);
        if (c=='s') this.add_yy1(1);
        if (c=='d') this.add_yy1(-1);
        if (c=='f') this.add_yy1(-10);
        c=getch();
  }
}
void Window::add_xx1(int x)
{       xx1+=x;
        xx2+=x;
        this.fdraw();
}
void Window::add_yy1(int y)
{       yy1+=y;
        yy2+=y;
        this.fdraw();
}
/***********************************************************************
 *                                                                     *
 *      Twindow Class member functions and support functions           *
 *                                                                     *
 *       setup()                Called by constructors for common code *
 *       print(char c)          Print a charater to text window        *
 *       print(char *line)      Print a C style character arrary       *
 *       print(String line)     Print a String to text window          *
 *                                                                     *
 ***********************************************************************/

void Twindow::setup()
{  rrow=0;
   ccol=0;
   op.txo_fgcolor.v=Wcp.char_in_win;
   op.txo_bgcolor.v=Wcp.window;
   r.txr_font=op.txo_font;
   int pixels_wide=window.ix2()-window.ix1();
   int pixels_high=window.iy2()-window.iy1();
   r.txr_width=pixels_wide/GrFontWidth(&op);
   r.txr_height=pixels_high/GrFontHeight(&op);
   maxrow=r.txr_height;
   maxcol=r.txr_width;
   r.txr_buffer=
       new(char [(r.txr_height+1)*r.txr_width+1]);
   r.txr_backup=
       new(char [r.txr_height*r.txr_width+1]);
   r.txr_lineoffset=r.txr_width;
   r.txr_fgcolor.v=op.txo_fgcolor.v;
   r.txr_bgcolor.v=op.txo_bgcolor.v;
   r.txr_chrtype=GR_BYTE_TEXT;
   r.txr_xpos=window.ix1()+
              (pixels_wide-
               r.txr_width*GrFontWidth(&op))/2;
   r.txr_ypos=window.iy1()+
              (pixels_high-
               r.txr_height*GrFontHeight(&op))/2;
   // initalize buffer to spaces
   for (int i=0; i<(r.txr_height+1) ; i++)
     for (int j=0; j<r.txr_width ; j++)
           set_c(i,j,' ');
}


void Twindow::print(char c)
{ if (c=='\n')
  { rrow++;
    ccol=0;
    if (rrow>maxrow)
    {  for (int i=0; i<(maxrow);i++)
       {   for (int j=0; j<maxcol ; j++)
              set_c(i,j,get_c(i+1,j));
       }
       rrow=maxrow;
    }
    GrDumpTextRegion(&r);
    return;
  }
  if (++ccol<maxcol)
  { set_c(rrow,ccol,c);       
    GrDumpChar(c,ccol,rrow,&r);
  }
}
void Twindow::print(char *line)
{    for (int i=0; i<strlen(line) ; i++)
     this.print(line[i]);
}
void Twindow::print(String line)
{    this.print( (char *) line);
}
/*****************************************************************************
 *                                                                           *
 *  Class Button                                                             *
 *                                                                           *
 *      Public Object                                                        *
 *              window of Window class                                       *
 *      Functions:                                                           *
 *              Button(int x1,int y1,int x2,int y2,String lable,             *
 *                      int disp=YES,int acti=YES,String fam="times")        *
 *                            -  Constructor                                 *
 *                            x1 -- top left corner x                        *
 *                            y1 -- top left corner y                        *
 *                            x2 -- bottom right corner x                    *
 *                            y2 -- bottom right corner y                    *
 *                            lable -- button lable                          *
 *                            disp -- diplay, YES or NO                      *
 *                            acti -- active, YES or NO                      *
 *                            fam -- Font Family                             *
 *              ~Button()     - Destructor                                   *
 *              upcolor()     - Set colors to up position                    *
 *              downcolor()   - Set colors to down position                  *
 *              active(int acti) - Change active status.                     *
 *              press()       - Displays pressed button and calles action    *
 *              action()      - Null Function for overriding                 *
 *              locate(int x1,int y1,int x2,int y2) - Locates position for   *
 *                              button text.                                 *
 *              cdraw()       - Context Draw                                 *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
Button::Button(int x1,int y1,int x2,int y2,String lable,
                        int disp,int acti,String fam) : 
            Box(x1, y1, x2, y2, Wcp.button_shadow),
            llable(lable),
            family(fam)
          { display=disp;
            active=acti;
            next=NULL;
            len=llable.length();
            if (len>0)
               this.locate(x1,y1,x2,y2);
            if (display==YES)
                this.upcolor();
          }
virtual void Button::upcolor()
          { if (display==NO) return;
            button_c.fbx_intcolor=Wcp.button_face;
            button_c.fbx_topcolor=Wcp.button_highlight;
            button_c.fbx_bottomcolor=Wcp.button_shadow;
            button_c.fbx_leftcolor=Wcp.button_highlight;
            button_c.fbx_rightcolor=Wcp.button_shadow;           
            this.fdraw();
          }
void Button::set_active(int acti)
          {  if (active==acti)
                return;
             active=acti;
             this.fdraw();   
          }
virtual void Button::downcolor()
          { if (display==NO) return;
            button_c.fbx_intcolor=Wcp.button_face;
            button_c.fbx_topcolor=Wcp.button_face;
            button_c.fbx_bottomcolor=Wcp.button_face;
            button_c.fbx_leftcolor=Wcp.button_face;
            button_c.fbx_rightcolor=Wcp.button_face;           
            this.fdraw();
          }
void Button::fdraw()
          {  if (display==NO) return;
             GrFramedBox(xx1,yy1,xx2,yy2,(xx2-xx1)/SPACING,&button_c);
             if (len>0)
                GrDrawString((char*)llable,len,
                                l_x, l_y, &op );
          }
virtual void Button::press()
          {  if (active==NO) return;
             this.downcolor();
             MouseGetEvent((M_BUTTON_UP ),&evt);
             this.upcolor();
             this.action();
          }
void Button::locate(int x1,int y1,int x2,int y2)
          {  // Given button coordinates change
             // lable coordiantes to match.
             // and load appropreate font.
             int max_width,max_height;
             int tot_width,tot_height;
             int flag=0;
             if (x2==-1)
                { flag=1; x2=4096; }               
             xx1=x1+1;
             yy1=y1+1;
             xx2=x2-1;
             yy2=y1+1+FONTHEIGHT;
                   
             max_width=(xx2-xx1)/len;
             max_height=yy2-yy1;
             if(NULL==GrFindBestFont(max_width,
                           max_height,
                           0,
                           (char *)family,
                           &op )
               )
               mess_i("Null from GrFind in button",0);
             op.txo_fgcolor.v=Wcp.button_text;
             op.txo_bgcolor.v=Wcp.button_face;
             tot_width=GrStringWidth((char *)llable,len,&op);
             tot_height=GrStringHeight((char *)llable,len,&op);
             if (flag==1)
             { xx2=xx1+tot_width+tot_width/SPACING; }
             l_x=xx1+(xx2-xx1-tot_width)/2;
             l_y=yy1+(yy2-yy1-tot_height)/2;
          }
/*****************************************************************************
 *                                                                           *
 *  Class ButtonList                                                         *
 *                                                                           *
 *      Public Object                                                        *
 *              Linked list of buttons that can be pressed.                  *
 *                                                                           *
 *      Functions:                                                           *
 *              ButtonList()               -Constructor initializes list     *
 *              void insert(Button *newb)  -Insert a button into the list    *
 *              Button *remove_first()     -Return pointer to first button   *
 *                                              and removes from list        *
 *              Button *find(int x,int y)  -Find the button at x,y           *
 *              void press(int x,int y)    -Press the button at x,y          *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
Button *ButtonList::remove_first()
        {   Button *tmp;
            if (head==NULL)
               return 0;
            tmp=head;
            head=tmp->next;
            return(tmp);
        }
Button *ButtonList::find(int x,int y)
        {       Button *tmp;
                for (tmp=head; tmp!=NULL ; tmp=tmp->next)
                {  if ( tmp->x1()<x && x<tmp->x2() &&
                        tmp->y1()<y && y<tmp->y2() )
                        return (tmp);
                }
                return NULL;
        }
void ButtonList::fdraw()
        {       Button *tmp;
                for (tmp=head; tmp!=NULL ; tmp=tmp->next)
                {  tmp->fdraw();
                }
                return;
        }

void ButtonList::press(int x,int y)
        {       Button *tmp=this.find(x,y);
                if (tmp==NULL)
                        return;
                tmp->press();
                
        }
/*****************************************************************************
 *                                                                           *
 *  Class Pointer                                                            *
 *                                                                           *
 *      Public Object                                                        *
 *              Takes care of mouse initalization and deinitialization       *
 *                                                                           *
 *      Functions:                                                           *
 *              Pointer()  - Constructor Initializes the mouse.              *
 *              ~Pointer() - Destructor Deinializes the mouse.               *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
 Pointer::Pointer ()
                {ok=FAIL;
                 if (MouseDetect())
                 {      MouseEventMode(1);
                        MouseInit();
                        MouseSetColors(color_table.c("RED"),
                                color_table.c("BLACK"));
                        MouseDisplayCursor();
                        MouseSetCursorMode(M_CUR_NORMAL);
                        MouseEventEnable(0,1);
                        ok=SUCESS;
                 }
                }
  Pointer::~Pointer()
                {
                 if (MouseDetect())
                        MouseUnInit();
                }
/*****************************************************************************
 *                                                                           *
 * initialize()                                                              *
 *      Funtion to initalize the public objects.                             *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
int initialize()
{
      ifstream color_file("COLOR.DAT");
      ifstream pref_colors("PCOL.DAT");

      if (color_file && pref_colors)
      {    color_file >> color_table;
           pref_colors>> Wcp;
           GrSetFontPath("C:\\DJGPP\\CONTRIB\\LIBGRX\\FONTS\\");  
           return SUCESS;
      }
      return FAIL;
}
