/*****************************************************************************
 *                                                                           *
 *      Window.h                                                             *
 *                                                                           *
 *      Combined from Window.h, Graphic.h, Debug.h, and ColorSel.h           *
 *            Includes the following Class Definitions                       *
 *                                                                           *
 *      Graphic         - Basic graphic class, initalizes graphics mode      *
 *                          and provides basic context information           *
 *      Color_Table     - Stores and retrives colors from the color table    *
 *                          by name.                                         *
 *      Box             - Basic Graphic element used to derive other classes *
 *      Color_Selector  - Displays color combinations to allow programer to  *
 *                          make the initialization file for color table     *
 *      Title_Bar       - Title bar for the window class                     *
 *      Window          - Basic window class, draws resizable, relocatable   *
 *                          window.                                          *
 *      Twindow         - Basic Text window with print functions             *
 *      Button          - Basic Button.                                      *
 *      ButtonList      - Linked list of buttons.                            *
 *      Pointer         - Mouse initalization                                *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
#ifndef __WINDOW_H
#define __WINDOW_H

#define FAIL 0
#define SUCESS 1
#define TRUE 1
#define FALSE 0
#define NO 0
#define YES 1
#define SPACING 20
#define FONTHEIGHT MaxY/30
#include <grx.h>
#include <mousex.h>
#include <iostream.h>    
#include <Strclass.h>
#include <fstream.h>
#include <stdio.h>
#include <conio.h>
/*****************************************************************************
 *                                                                           *
 *  Message functions used for error reporting and debugging                 *
 *                                                                           *
 *****************************************************************************/

void mess_i(String m,int val);
void mess_s(String m,char *val);
/*****************************************************************************
 *                                                                           *
 *  Graphic Class                                                            *
 *                                                                           *
 *      Functions                                                            *
 *              Graphic()   - Constructor initalizes graphics.               *
 *              ~Graphic()  - Destructor restores text mode.                 *
 *                                                                           *
 *              total_number_graphic_objects()  - returns the number of      *
 *                              graphic objects defined                      *
 *              scale()     - scale, returns the scale number. (not used yet)*
 *                                                                           *
 *              set_context(GrContext *new_context) - Sets a new graphic     *
 *                              context for the graphic object               *
 *              context()   - returns the graphic context of the object      *
 *              maxx()      - returns the max number of pixels in the x dir  *
 *              maxy()      - returns the max number of pixels in the y dir  *
 *              maxc()      - returns the max number of colors allowed       *
 *                                                                           *
 *****************************************************************************/

class Graphic
{       protected:

        static int initialized;
        static int ccount;
        static int MaxX,MaxY,MaxC;
        static float sscale;  
        // ratio of realpixel / MAX_Y pixels
        GrContext *ccontext;
        
        public:
        
        Graphic()
        {  if (initialized==0)
           {    GrSetMode(GR_biggest_noninterlaced_graphics); 
                initialized=1;
                MaxX=GrMaxX();
                MaxY=GrMaxY();
                MaxC=GrNumColors();
           }
           ccontext=NULL;
           ccount++;

        }
        ~Graphic()
        {  ccount--;
           if (ccount==0)
                GrSetMode(GR_default_text);
        }
        inline int total_number_graphic_objects() { return ccount; };
        inline float scale() { return sscale; };
        inline void set_context(GrContext *new_context)
                { ccontext=new_context; };
        inline GrContext* context() {return ccontext; };
        inline int maxx() { return MaxX; };
        inline int maxy() { return MaxY; };
        inline int maxc() { return MaxC; };
};

#ifdef __WINDOW_CC
        int Graphic::ccount=0;
        int Graphic::initialized=0;
        int Graphic::MaxX=0, Graphic::MaxY=0, Graphic::MaxC=0;
        float Graphic::sscale=0;
#endif
/*****************************************************************************
 *                                                                           *
 *  class Color Table                                                        *
 *                                                                           *
 *        Supporting Structures                                              *
 *          color_info                                                       *
 *        Functions                                                          *
 *          operator>>          -  Operator to read color table data from a  *
 *                                      file.                                *
 *          alloc(int r, int g, int b, String name)     - allocate a color   *
 *                                                        into the color     *
 *                                                        table with name.   *
 *          c(String name)      -  Return the color number of the named color*
 *                                                                           *
 *****************************************************************************/

#define MAX_COLORS 254
struct color_info
{       String color_name;
        int color_number;
} ;

class Color_Table : public Graphic
{
        private :
           color_info c_table[ MAX_COLORS ] ;
           int entries;
        public :
           friend istream& operator>> (istream& s, Color_Table& c);
           Color_Table();
           int alloc(int r, int g, int b, String name) 
           { 
             if (entries>=MaxC) 
                return FAIL;
             c_table[entries].color_number=GrAllocColor(r,g,b);
             c_table[entries].color_name=name;
             entries++;
             return SUCESS;
           }
           int c(String name)
           {    for (int i=0 ; i<entries; i++)
                {  if (c_table[i].color_name==name)
                     return c_table[i].color_number;
                }
                return -1;
           }
};
istream& operator>> (istream& s, Color_Table& c);
#ifdef __WINDOW_CC
Color_Table color_table;
#else
extern Color_Table color_table;
#endif
/*****************************************************************************
 *                                                                           *
 *  class Box                                                                *
 *                                                                           *
 *      Functions                                                            *
 *              operator<<      - Print box information                      *
 *              Box()           - Constructor                                *
 *              Box(int x1, int y1, int x2, int y2, int c)  - Constructor    *
 *                                      with position and color info         *
 *              cdraw()         - context draw funtion                       *
 *              fdraw()         - fast draw funtion                          *
 *              set_c(int new_c)   - set color                               *
 *              c()             - return color                               *
 *              x1()            - return x1 top left x coordinate            *
 *              y1()            - return y1 top left y coordinate            *
 *              x2()            - return x2 bottom right x coordinate        *
 *              y2()            - return y2 bottom right y coordinate        *
 *              add_x( int added_x)     - add integer to x1                  *
 *              add_y( int added_y)     - add integer to x2                  *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
class Box : public Graphic
{ 
        protected:
                int cc;  
                int xx1, yy1, xx2, yy2;
        public:
                friend ostream& operator<< (ostream& s, const Box& b) ;
                Box() :
                  Graphic()
                { xx1=0; yy1=0;
                  xx2=MaxX/12; yy2=MaxY/8;
                  cc=GrWhite(); 
                }
                Box(int x1, int y1, int x2, int y2, int c) :
                  Graphic()
                { xx1=x1; yy1=y1; xx2=x2; yy2=y2; cc=c; }
                virtual void cdraw()
                { GrSetContext(ccontext);
                  GrBox(xx1,yy1,xx2,yy2,cc);
                }
                inline virtual void fdraw() { GrBox(xx1,yy1,xx2,yy2,cc); };
                inline void set_c(int new_c) {cc=new_c;};
                inline int  c() { return cc; };
                inline int  x1() { return xx1; };
                inline int  y1() { return yy1; };
                inline int  x2() { return xx2; };
                inline int  y2() { return yy2; };
                void add_x( int added_x) ;
                void add_y( int added_y) ;
};
ostream& operator<< (ostream& s, const Box& b);
/*****************************************************************************
 *                                                                           *
 *  class Color Selector                                                     *
 *                                                                           *
 *      setup()                 - called by constructor for common code      *
 *      Color_Selector()        - Default constructor                        *
 *      Color_Selector(int x1, int y1, int x2, int y2, int c) - Constructor  *
 *      cdraw()                 - Context Draw                               *
 *      fdraw()                 - Fast Draw                                  *
 *      add_x(int added_x)      - add integer to x coordinate                *
 *      add_y(int added_y)      - add integer to y coordinate                *
 *      add_r(int added_r)      - add integer to red value                   *
 *      add_g(int added_g)      - add integer to green value                 *
 *      add_b(int added_b)      - add integer to blue value                  *
 *      void cset()             - resets color of color cell                 *
 *      adjust()                - Interactively adjust color                 *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
#define Font_Hight 16
#define Font_Width 8

class Color_Selector : public Box
{
        protected:
                GrFBoxColors c1,c2;
                int color_cell;
                int r,g,b,w;
                int xx3,yy3,xx4,yy4;
                int bg;
                int top_left1x, top_left2x, top_left3x;
                int top_lefty;
        public :
                void setup();
                Color_Selector() : 
                  Box    (MaxX/10,   // x1
                          MaxY/8,    // y1
                          MaxX/8*2,  // x2
                          MaxY/8*2,  // y2
                          GrWhite())     // c
                  {  r=g=b=125;
                     this.setup(); 
                  }
                Color_Selector(int x1, int y1, int x2, int y2, int c) :
                  Box(x1,y1,x2,y2,c)
                  {  r=g=b=125;
                     this.setup(); 
                  }
                void cdraw();
                void fdraw() { this.cdraw(); };
                void add_x(int added_x);                        
                void add_y(int added_y);
                void add_r(int added_r);
                void add_g(int added_g);
                void add_b(int added_b);
                inline void cset() {GrSetColor(color_cell,r,g,b);}
                void adjust();
};
/*****************************************************************************
 *                                                                           *
 *  Structure Win Colors                                                     *
 *      Structure to hold color prefrences for window displays.              *
 *                                                                           *
 *      operator>>      - reads window color prefrence file and loads        *
 *                              Win_Colors Structure.                        *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
struct Win_Colors
{       int background;
        int window;
        int window_frame;
        int char_in_win;
        int scroll_bar;
        int menu;
        int char_in_menu;
        int active_title_bar;
        int inact_title_bar;
        int char_act_title;
        int char_ina_title;
        int popup_win;
        int char_in_popup;
        int message_window;
        int char_in_message;
        int button_face;
        int button_text;
        int button_shadow;
        int button_highlight;
}; 
#ifdef __WINDOW_CC
Win_Colors Wcp;
#else
extern Win_Colors Wcp;
#endif
//
//      >> Operator to read Win Colors.
//
istream& operator>> (istream& s, Win_Colors& Wcp);
/*****************************************************************************
 *                                                                           *
 *  Class Title_Bar                                                          *
 *                                                                           *
 *      Functions:                                                           *
 *              Title_Bar(char *m,char *fam)    - Constructor,               *
 *                                                 message, and font family  *
 *              cdraw()         -  Context draw                              *
 *              fdraw()         -  Fast draw                                 *
 *              locate(int x1,int y1,int x2,int y2) - given window location  *
 *                                  place the title bar                      *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
class Title_Bar : public Box
{
        protected: 
                int m_x,m_y;     // m=message
                String message;
                int len;
                String family;
                GrTextOption op;
                GrFont *font;
        public:
                Title_Bar(char *m,char *fam) :
                        message(m),
                        family(fam),
                        Box()  
                {  
                   len=message.length();
                   cc=Wcp.active_title_bar;  // Set Bar Color.
                }
                void cdraw()
                {  
                   GrSetContext(ccontext);
                   this.fdraw();
                }
                inline void fdraw()
                {  GrFilledBox(xx1,yy1,xx2,yy2,cc);
                   GrDrawString((char*)message,len,
                                m_x, m_y, &op );
                }
                void locate(int x1,int y1,int x2,int y2);
};
/*****************************************************************************
 *                                                                           *
 *  Class Window                                                             *
 *                                                                           *
 *      Functions:                                                           *
 *              Window (String name)                                         *
 *              Window (int x1,int y1,int x2,int y2,String name)             *
 *              cdraw()         -  Context draw                              *
 *              fdraw()         -  Fast draw                                 *
 *              ix1()           -  return internal x1 coordinate             *
 *              iy1()           -  return internal y1 coordinate             *
 *              ix2()           -  return internal x2 coordinate             *
 *              iy2()           -  return internal y2 coordinate             *
 *              resize()        -  interactively resize window               *
 *              add_xx1(int x)  -  add to x1 coordinate                      *
 *              add_yy1(int y)  -  add to y1 coordinate                      *
 *              move()          -  interactively move window location        *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
class Window : public Box
{       protected :
                Title_Bar title;
                GrFBoxColors win_frame_c;
                int iix1,iiy1,iix2,iiy2; // internal offsets.
        public :
        Window (String name) :
          Box(MaxX/4,
              MaxY/4,
              MaxX/4*3,
              MaxY/4*3,
              Wcp.window_frame),
          title(name,"times")
        {    win_frame_c.fbx_intcolor=Wcp.window;
             win_frame_c.fbx_topcolor=Wcp.window_frame;
             win_frame_c.fbx_bottomcolor=Wcp.window_frame;
             win_frame_c.fbx_leftcolor=Wcp.window_frame;
             win_frame_c.fbx_rightcolor=Wcp.window_frame;
             title.locate(xx1,yy1,xx2,yy2);
             iix1=0; iiy1=1; iix2=-1; iiy2=-1;
        }           
        Window (int x1,int y1,int x2,int y2,String name) :
          Box(x1,y1,x2,y2,Wcp.window_frame),
          title(name,"times")
        {    win_frame_c.fbx_intcolor=Wcp.window;
             win_frame_c.fbx_topcolor=Wcp.window_frame;
             win_frame_c.fbx_bottomcolor=Wcp.window_frame;
             win_frame_c.fbx_leftcolor=Wcp.window_frame;
             win_frame_c.fbx_rightcolor=Wcp.window_frame;
             iix1=0; iiy1=1; iix2=-1; iiy2=-1;
        }           
        void cdraw()
        {   GrSetContext(ccontext);
            this.fdraw();
        }
        // return internal window drawing limits
        inline int ix1() { return title.x1()+iix1; };
        inline int iy1() { return title.y2()+iiy1; };
        inline int ix2() { return this.x2()+iix2; };
        inline int iy2() { return this.y2()+iiy2; };

        // add to internal window drawing limits
        inline void add_ix1(int x1) { iix1+=x1; };
        inline void add_iy1(int y1) { iiy1+=y1; };
        inline void add_ix2(int x2) { iix2+=x2; };
        inline void add_iy2(int y2) { iiy2+=y2; };

        void fdraw()
        {  GrClearScreen(Wcp.background);
           GrFramedBox(xx1,yy1,xx2,yy2,MaxX/100,&win_frame_c);
           title.locate(xx1,yy1,xx2,yy2);
           title.cdraw();
        }
        void resize();
        void add_xx1(int x);
        void add_yy1(int y);
        void move();
};
/*****************************************************************************
 *                                                                           *
 *  Class Twindow                                                            *
 *                                                                           *
 *      Functions:                                                           *
 *              ~Twindow()      -  Destructor                                *
 *              Twindow(String name,String fam)  - Constructor name, fontfam *
 *              Twindow(String name,String fam,                              *
 *                      int font_width,int font_height)                      *
 *                                      - Constructor name, fontfam & font   *
 *                                               size                        *
 *              cdraw()         - Context Draw                               *
 *              print(char c);  - Print a character                          *
 *              print(char *line);  - Print a character array                *
 *              print(String line); - Print a String                         *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
#define DEFAULT_WIDTH 9
#define DEFAULT_HEIGHT 16
class Twindow : Box
{       protected :
                GrTextRegion r;
                GrTextOption op;
                int rrow,ccol,maxrow,maxcol;
                String family;
                inline void set_c(int row,int col,char c)
                {   r.txr_buffer
                        [row*r.txr_lineoffset+col]=c;
                };
                inline char get_c(int row,int col)
                {   return (r.txr_buffer
                                [row*r.txr_lineoffset+col]);
                };
                void setup();
        public :
                Window window;
                ~Twindow()
                {       delete r.txr_buffer;
                        delete r.txr_backup;
                }
                Twindow(String name,String fam) :
                  window(name),
                  family(fam)
                { if(NULL==GrFindBestFont(DEFAULT_WIDTH,
                                          DEFAULT_HEIGHT,0,
                                  (char *) family,
                                  &op )
                      )
                       mess_i("Null returned from GrFind",0);
                  this.setup();
                }         
                Twindow(String name,String fam,
                        int font_width,int font_height) :
                  window(name),
                  family(fam)
                { if(NULL==GrFindBestFont(font_width,
                                          font_height,0,
                                  (char *) family,
                                  &op )
                      )
                       mess_i("Null returned from GrFind",0);
                  this.setup();
                }         
                void cdraw()
                {       window.cdraw();
                        GrDumpTextRegion(&r);
                }
                void print(char c);
                void print(char *line);
                void print(String line);
}; 
/*****************************************************************************
 *                                                                           *
 *  Class Button                                                             *
 *                                                                           *
 *      Functions:                                                           *
 *              Button(int x1,int y1,int x2,int y2,String lable,             *
 *                      int disp=YES,int acti=YES,String fam="times")        *
 *                            -  Constructor                                 *
 *                            x1 -- top left corner x                        *
 *                            y1 -- top left corner y                        *
 *                            x2 -- bottom right corner x                    *
 *                            y2 -- bottom right corner y                    *
 *                            lable -- button lable                          *
 *                            disp -- diplay, YES or NO                      *
 *                            acti -- active, YES or NO                      *
 *                            fam -- Font Family                             *
 *              ~Button()     - Destructor                                   *
 *              upcolor()     - Set colors to up position                    *
 *              downcolor()   - Set colors to down position                  *
 *              active(int acti) - Change active status.                     *
 *              press()       - Displays pressed button and calles action    *
 *              action()      - Null Function for overriding                 *
 *              locate(int x1,int y1,int x2,int y2) - Locates position for   *
 *                              button text.                                 *
 *              cdraw()       - Context Draw                                 *
 *              fdraw()       - Fast Draw                                    *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
class Button  : public Box
{
       protected :
          GrFBoxColors button_c;
          int len;
          String llable,family;
          int l_x,l_y;     // l=lable
          GrTextOption op;
          MouseEvent evt;
          GrFont *font;
       public:
          Button *next;
          int active,display;

          Button(int x1,int y1,int x2,int y2,String lable,
                        int disp=YES,int acti=YES,String fam="times") ; 
          virtual void upcolor() ;
          void set_active(int acti=YES) ;
          virtual void downcolor() ;
          void fdraw() ;
          virtual void press() ;
          virtual void action()
          {  // do nothing.
          }
          void locate(int x1,int y1,int x2,int y2) ;
};
/*****************************************************************************
 *                                                                           *
 *  Class ButtonList                                                         *
 *                                                                           *
 *      Public Object                                                        *
 *              Linked list of buttons that can be pressed.                  *
 *                                                                           *
 *      Functions:                                                           *
 *              ButtonList()               -Constructor initializes list     *
 *              void insert(Button *newb)  -Insert a button into the list    *
 *              Button *remove_first()     -Return pointer to first button   *
 *                                              and removes from list        *
 *              Button *find(int x,int y)  -Find the button at x,y           *
 *              void press(int x,int y)    -Press the button at x,y          *
 *      
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
class ButtonList  
{
        Button *head;

        public:

        ButtonList() { head=NULL; } 
        inline void insert(Button *newb)
        {   newb->next=head;
            head=newb;
        };
        Button *remove_first() ;
        Button *find(int x,int y)  ;
        void press(int x,int y)  ;
        void fdraw();
};
/*****************************************************************************
 *                                                                           *
 *  Class Pointer                                                            *
 *                                                                           *
 *      Public Object                                                        *
 *              Takes care of mouse initalization and deinitialization       *
 *                                                                           *
 *      Functions:                                                           *
 *              Pointer()  - Constructor Initializes the mouse.              *
 *              ~Pointer() - Destructor Deinializes the mouse.               *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/

class Pointer
{       public :
                MouseEvent evt; 
                int ok;
                Pointer ();
                ~Pointer();
};
/*****************************************************************************
 *                                                                           *
 * initialize()                                                              *
 *      Funtion to initalize the public objects.                             *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/

int initialize();
#endif
